#!/usr/bin/env python3
""" GSSw Hardware Abstraction Layer Manager 
"""

from gssw.common             import initGssw
from gssw.lib.application    import Application
from gssw.hal.hal_frontend   import FrontEnd
from gssw.lib.misc           import createGsswArgumentParser
from gssw.lib.sockets        import destroyAllSocketsInContext

def main():
    """ Starts the HalManager. """

    # Parse the command line arguments containing the config server address or config file path
    args = createGsswArgumentParser(__doc__, enableOptionName=True,
                                    optionNameDefault='hal_manager').parse_args()

    config      = initGssw(args)
    application = Application(config, name="Hardware Abstraction Manager")
    gui         = FrontEnd(config, application)

    try:
        gui.show()
        application.exec_()
    finally:
        destroyAllSocketsInContext()

if __name__ == "__main__":
    main()
