#!/usr/bin/env python3

""" This module starts the hk manager. """

from gssw.common             import initGssw
from gssw.hk.hkmanager       import HkManager
from gssw.lib.misc           import createGsswArgumentParser
from gssw.lib.application    import Application
from gssw.lib.sockets        import destroyAllSocketsInContext

def main():
    """ Starts the hk manager.
    """
    # Parse the command line arguments containing the config server address or config file path
    args = createGsswArgumentParser(__doc__, enableOptionName=True,
                                    optionNameDefault='hk_manager').parse_args()

    config      = initGssw(args)
    application = Application(config, name="House Keeping Manager")
    gui         = HkManager(args.name, config, application)

    try:
        gui.show()
        application.exec_()
    finally:
        destroyAllSocketsInContext()


if __name__ == "__main__":
    main()
