#! /usr/bin/env python3

""" Component that starts a GUI for quicklook of measurement data
"""

import logging
import sys

from gssw.common import initGssw
from gssw.quicklook.main import PROGRAM_NAME, MainWindow

from gssw.lib.application import Application
from gssw.lib.misc import createGsswArgumentParser


logger = logging.getLogger(__name__)


def main():
    """ Starts the quicklooker.
    """
    logger.info('Starting quicklooker')

    # Parse the command line argument(s)
    parser = createGsswArgumentParser(__doc__, enableOptionName=True,
                                      optionNameDefault='quicklooker')
    parser.add_argument('--reset', action='store_true',
                        help='Resets persistent settings (e.g. window positions)')

    parser.add_argument('-b', '--bind-sub-socket', dest='bindTo',
                        help="Also binds the subsocket to this address. Useful for testing. "
                             "'Can be a full address such as 'tcp://*:5555', or an integer "
                             "in which case it is interpreted as a port number and the sub-socket "
                             "is bound to 'tcp://*:<portnr>'")
                             
    parser.add_argument('-c', '--connect-sub-socket',
                        help="Also connects the subsocket to this address. Useful for testing. "
                             "'Can be a full address such as 'tcp://hostname:5555'")

    args = parser.parse_args()
    config = initGssw(args)

    app = Application(config, name=PROGRAM_NAME, style=True)
    mainWindow = MainWindow.createFromConfig(args.name, config, reset=args.reset)

    app.signalHandler.sigInt.connect(mainWindow.quitApplication)
    app.signalHandler.sigTerm.connect(mainWindow.quitApplication)

    if args.bindTo:
        try:
            port = int(args.bindTo)
            address = "tcp://*:{:d}".format(port)
        except ValueError:
            address = args.bindTo
        logger.debug("Binding sub-socket to: {}".format(address))
        mainWindow.bindSubSocket(address)

    if args.connect_sub_socket:
        address = args.connect_sub_socket
        logger.error("Connecting sub-socket to: {}".format(address))
        mainWindow.connectSubSocket(address)

    mainWindow.show()



    # Raise the window under OS-X
    if 'darwin' in sys.platform:
        mainWindow.raise_()

    # socketParams = params['sockets']
    #
    # mainWindow.bindRepSocket(makeAddress(socketParams['ql-rep'], host='*'))
    # mainWindow.bindSubSocket(makeAddress(socketParams['ql-sub'], host='*'))
    try:
        app.exec_()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':

    main()

