#!/usr/bin/env python3
""" GSSw GetSet Manager
"""

from gssw.common                 import initGssw
from gssw.lib.application        import Application
from gssw.setget.setget_frontend import FrontEnd
from gssw.lib.misc               import createGsswArgumentParser
from gssw.lib.sockets            import destroyAllSocketsInContext

def main():
    """ Starts the setGetManager.
    """
        
    # Parse the command line arguments containing the config server address or config file path
    parser = createGsswArgumentParser(__doc__, enableOptionName=True,
                                      optionNameDefault='set_get_manager')

    # Parse the command line arguments
    args        = parser.parse_args()
    config      = initGssw(args)
    application = Application(config, name='Set Get Manager')
    gui         = FrontEnd(args.name, config, application)

    try:
        gui.show()
        application.exec_()
    finally:
        destroyAllSocketsInContext()
   
   
if __name__ == "__main__":
    main()
