#!/usr/bin/env python3
""" Startup script for tray monitoring utility """

from gssw.lib.misc                import createGsswArgumentParser
from gssw.traymonitor.traymonitor import App
from gssw.lib.widgets.misc import gsswUiError
from PyQt5.QtWidgets import QApplication

parser = None  #pylint: disable=invalid-name

def visualExit(status=0, message=None):
    """ Exit with Qt popup message.
    
    The reason for this is that when the application is started from the Debian menu, stdout is
    not displayed. The user cannot see error messages on the console.

    :param int status:  System exit code.
    :param str message: Error message.
    """
    import sys
    if message is not None:
        from io import StringIO
        fresult = StringIO()

        # must remember object, otherwise segfault when showing dialog
        app = QApplication([]) # pylint: disable=unused-variable

        # Extract help information
        parser.print_help(fresult)
        helpText = fresult.getvalue()
        helpText = helpText.replace('\n', '<br />')


        gsswUiError('<b>Could not start Egress traymonitor</b><br /><br />'
                    '<b>Reason:</b> {}<br /><br /><b>Help:</b><br />{}'.format(
            message, helpText))
    sys.exit(status)


def main():
    """ Starts the tray monitoring program """

    print("Starting tray monitoring utility...")
    global parser  #pylint: disable=global-statement, invalid-name
    parser = createGsswArgumentParser(__doc__)
    parser.add_argument('--nosplash', help='Do not show the splash screen', action='store_true')
    parser.exit = visualExit # Use custom exit method

    args = parser.parse_args()

    app = App(args)
    app.run()

if __name__ == "__main__":
    main()
