""" HDF5 client for recording data using the python HDF5 datawriter

"""
import logging
from gssw.datawriter.baseclient import Hdf5BaseClient
import gssw.proto.hdf5writer_pb2    as      h5proto
from gssw.proto.common_pb2 import OkRep

logger = logging.getLogger(__name__)

class Hdf5GenericClient(Hdf5BaseClient):
    """ Hdf5GenericClient client class """

    def registerArray(self, arrayName, h5Path, concatDim=0):
        """ Listen for GriddedArrays of this name and write them to the dataset having the h5Path.

            Remotely calls :meth:`gssw.datawriter.h5worker.H5ArchiverWorker.registerArray`

            :param arrayName: any received array having this name will be written to the archive
            :param h5Path: the result will be appended to this HDF-5 dataset. The full path
                name should be specified.
            :param concatDim: dimension along which newly received data will be concatenated.
        """
        requestMsg = h5proto.RegisterArrayReq(arrayName=arrayName, h5Path=h5Path, 
                                                concatDim=concatDim)
        self.requestReply(requestMsg, OkRep())


    def unregisterArray(self, arrayName):
        """ Unregisters the Gridded array with name arrayName.

            Remotely calls :meth:`gssw.datawriter.h5worker.H5ArchiverWorker.unregisterArray`
        """
        requestMsg = h5proto.UnregisterArrayReq(arrayName = arrayName)
        self.requestReply(requestMsg, OkRep())


    def appendArray(self, array):
        """ Appends a gridded array to the corresponding dataset.

            Remotely calls :meth:`gssw.datawriter.h5worker.H5ArchiverWorker.appendArray`

            :param GriddedArray array: A gridded array with data to append.

        """
        requestMsg = h5proto.AppendArrayReq()
        array.addToProtoBuf(requestMsg.array)
        self.requestReply(requestMsg, OkRep())

