#!/usr/bin/env python3
""" Contains the release version. This version is only to be changed
    up on official releases!

    * MAJOR: GSSw wide updates
    * MINOR: important new features
    * PATCH: bug fixes

    Initial release for the GSSw is 2.0.0

    This version must be used by all GSSw components to indicate the
    release version (in log output, application bar, about, ...

    Typical usage:

    from gssw.lib.version import VERSION_STRING

    class myClass():
        def __init__(self):

            ...
            logger.info("Release version %s", VERSION_STRING)

"""

import argparse

MAJOR = 2
MINOR = 2
PATCH = 3

#: The version number of Egress.
VERSION_NUMBER = (MAJOR << 16) + (MINOR << 8) + PATCH
#: The version number of Egress, in displayable format.
VERSION_STRING = "%d.%d.%d" % (MAJOR, MINOR, PATCH)

def main():
    """ This file is executed by qmake for building the hwdaemon.
        The stdout is read as the version string. """
    parser = argparse.ArgumentParser(description="Script to print the version number to stdout.")
    parser.add_argument('--generate', type=str, choices=['MINOR', 'PATCH'], default=None,
        help='Generate an new version number (increment minor or patch by 1), and print it. '
             'Only used in scripts related to debian packaging. '
             'Note: the version number stored in this file is NOT modified.')

    args = parser.parse_args()
    if not args.generate:
        # The stdout is read as the version string
        print (VERSION_STRING)
    elif args.generate=='MINOR':
        print(MINOR + 1)
    elif args.generate=='PATCH':
        print(PATCH + 1)

if __name__ == '__main__':
    main()
