# pylint: skip-file


import datetime as dt
import numpy as np

from PyQt5 import QtWidgets

from silx.gui.plot import Plot1D

if 0:
    BACKEND = 'gl' # this gives incorrect results
else:
    BACKEND = 'mpl' # this works



def sizePolicyStr(widget):
    return ("vertical={}, horizontal={}"
              .format(widget.sizePolicy().horizontalPolicy(),
                      widget.sizePolicy().verticalPolicy()))


class Legend(QtWidgets.QWidget):

    MIN_SIZE_HOR = 50
    MIN_SIZE_VER = 50

    def __init__(self, direction=None, names=None, parent=None):
        super().__init__(parent=parent)

        self._names = [] if names is None else names

        if direction is None:
            direction = QtWidgets.QBoxLayout.TopToBottom

        self.mainLayout = QtWidgets.QBoxLayout(direction)
        self.setLayout(self.mainLayout)

        for name in self.names:
            label = QtWidgets.QLabel(name)
            self.mainLayout.addWidget(label)

            print("  label {:10s}: {}".format(name, sizePolicyStr(label)))

        self.mainLayout.addStretch()
        self.setMinimumSize(self.MIN_SIZE_HOR, self.MIN_SIZE_VER)

        # print("before : {}".format(sizePolicyStr(self)))
        #
        # self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
        #                    QtWidgets.QSizePolicy.Expanding)
        #
        # print("after  : {}".format(sizePolicyStr(self)))



    @property
    def names(self):
        """ The list of labels. Should be considered read only.
        """
        return self._names


    def appendName(self, name):
        self.names.append(name)




class MyWindow(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.setWindowTitle("No splitter")


        self.mainLayout = QtWidgets.QHBoxLayout()
        self.setLayout(self.mainLayout)

        # Make large value with a relatively small range by creating POSIX time stamps.
        base = dt.datetime.today()
        dates = [base - dt.timedelta(seconds=x) for x in range(0, 2500, 20)]

        x = np.array([d.timestamp() for d in dates], dtype=np.uint64)
        np.random.seed(seed=1)
        y = np.random.random(x.shape) * 12 - 3

        print('x[0:10]: {}'.format(x[0:10]))

        plot1D = Plot1D(backend=BACKEND)
        xAxis = plot1D.getXAxis()
        curve = plot1D.addCurve(x=x, y=y, legend='curve', symbol='o')
        self.mainLayout.addWidget(plot1D)

        legend = Legend(names = ['Amsterdammer veen', 'Haarlem', 'Delft', 'Leiden'])
        self.mainLayout.addWidget(legend)



class MyWindow2(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.setWindowTitle("With splitter")

        self.mainLayout = QtWidgets.QHBoxLayout()
        self.setLayout(self.mainLayout)

        self.splitter = QtWidgets.QSplitter()
        self.mainLayout.addWidget(self.splitter)

        # Make large value with a relatively small range by creating POSIX time stamps.
        base = dt.datetime.today()
        dates = [base - dt.timedelta(seconds=x) for x in range(0, 2500, 20)]

        x = np.array([d.timestamp() for d in dates], dtype=np.uint64)
        np.random.seed(seed=1)
        y = np.random.random(x.shape) * 12 - 3

        print('x[0:10]: {}'.format(x[0:10]))

        plot1D = Plot1D(backend=BACKEND)
        xAxis = plot1D.getXAxis()
        curve = plot1D.addCurve(x=x, y=y, legend='curve', symbol='o')
        self.splitter.addWidget(plot1D)

        legend = Legend(names = ['Amsterdam', 'Haarlemmermeer polder', 'Delft', 'Leiden'])
        self.splitter.addWidget(legend)





def main2():
    app = QtWidgets.QApplication([])
    win = MyWindow2()
    #win = Legend(names = ['Amsterdammer veen', 'Haarlem', 'Delft', 'Leiden'])
    win.show()
    app.exec_()




if __name__ == "__main__":
    main()
