# pylint: skip-file

import logging
import sys
import numpy as np
import numpy.testing as npt

from gssw.proto.array_pb2 import NumpyArrayMsg

from gssw.lib.arrays.semantic import SemanticArray, concatenate as semConcat
from gssw.lib.arrays.gridded import GriddedArray, concatenate as gridConcat
from gssw.lib.rpcmsg import Packetizer

logger = logging.getLogger("main")



def main1():
    arrMsg = NumpyArrayMsg()
    arr1 = np.arange(6, dtype=np.float64) / 3.444
    arr2 = np.arange('2018-08-20T12:00Z', '2018-08-20T14:00Z', 6, dtype='datetime64[ms]')
    packetizer = Packetizer.createDefaultPacketizer()
    packetizer.setMessageAttrs(arrMsg, arr2)
    check = packetizer.getMessageAttrs(arrMsg)
    npt.assert_array_equal(check, arr2)
    logger.debug("check: {}".format(check.dtype.str))



def main2():

    x1 = np.array([], dtype='datetime64[ms]')
    #x1 = np.arange('2018-08-20T12:00Z', '2018-08-20T14:00Z', 560000, dtype='datetime64[ms]')
    x2 = np.arange('2019-08-20T12:00Z', '2019-08-20T14:00Z', 460000, dtype='datetime64[ms]')

    print(x1.shape)
    print(x2.shape)

    sem1 = SemanticArray(x1, name="x1", units='date')
    sem2 = SemanticArray(x2, name="x2", units='date')
    semAll = semConcat([sem1, sem2])

    y1 = np.arange(len(x1)) * 5.5
    y2 = np.arange(len(x2)) * -1.5

    grid1 = GriddedArray(y1, name="data1", coords={0:sem1})
    grid2 = GriddedArray(y2, name="data2", coords={0:sem2})
    gridAll = gridConcat([grid1, grid2])
    print(gridAll)


    
    
if __name__ == "__main__":
    FMT = '%(filename)25s:%(lineno)-4d : %(levelname)-7s: %(message)s'
    logging.basicConfig(level='DEBUG', stream=sys.stderr, format=FMT)
    # main1()
    main2()
    logger.info("DONE")
