#!/usr/bin/env python3
# pylint: skip-file

""" Try the silx package.

    export PYTHONPATH="/home/pepijnk/src/extern/silx:${PYTHONPATH}"


"""

import os
# for key, value in sorted(os.environ.items()):
#     print("{:30s}: {}".format(key, value))
#os.environ.setdefault('QT_API', 'pyqt4')

import sys, logging
import numpy as np
import datetime as dt
import dateutil.tz

from PyQt5 import QtWidgets
from silx.gui.plot import Plot1D, Plot2D, TickMode
from silx.gui.plot.Colormap import Colormap
from silx.gui.plot._utils.dtime_ticklayout import timestamp

logger = logging.getLogger("main")

if 0:
    BACKEND = 'mpl'
else:
    BACKEND = 'gl'



def makePlot1D():

    CREATE_DATES = True

    if not CREATE_DATES:
        y = np.random.normal(size=int(1e5))
        y[10000] = 23

        x = np.arange(len(y)) + 9000
        x = list(x)

    else:
        #x = np.arange('2018-03-20T12:00', '2018-03-23T12:00', 156, dtype='datetime64[m]')

        x = np.arange('2018-08-20T12:00Z', '2018-08-20T14:00Z', 14106, dtype='datetime64[ms]') # small number of points

        # x = np.arange('2017-12-31T12:00', '2018-01-02T14:00', 15, dtype='datetime64[s]')  # date time border
        # x = np.arange('1970-12-31T12:00', '1971-01-02T14:00', 15, dtype='datetime64[s]')  # date time border

        #x = np.arange('1970-12-31T05:57', '1970-12-31T06:00', 15, dtype='datetime64[s]')  # 1970

        # Just using np.datetime64 arrays doesn't work. It will give a TypeError:
        #   ufunc 'isnan' not supported for the input types, and the inputs could not be safely
        #   coerced to any supported types according to the casting rule ''safe''
        # See also https://github.com/numpy/numpy/issues/5610
        # Therefore we use tolist() to convert to an array of datetime objects
        #x = np.array(x.tolist())
        #x = np.array([timestamp(d) for d in x.tolist()], dtype=np.uint64)
        #x = [d.timestamp() for d in x.tolist()]
        #x = [d.timestamp() for d in x.tolist()]
        x = np.array(x.tolist())

        print("x.dtype: {}".format(x.dtype))

        #
        #
        # x = [dt.datetime(2018, 8, 20, 12, 0,  0, 0),
        #      dt.datetime(2018, 8, 20, 12, 0, 14, 106000),
        #      dt.datetime(2018, 8, 20, 12, 0, 28, 212000),
        #      dt.datetime(2018, 8, 20, 12, 0, 42, 318000),
        #      dt.datetime(2018, 8, 20, 12, 0, 56, 424000)]
        #
        # tz = dateutil.tz.tzutc()
        # x = [dt.datetime(2018, 8, 20, 12, 0,  0, 0, tzinfo=tz),
        #      dt.datetime(2018, 8, 20, 12, 0, 14, 106000, tzinfo=tz),
        #      dt.datetime(2018, 8, 20, 12, 0, 28, 212000, tzinfo=tz),
        #      dt.datetime(2018, 8, 20, 12, 0, 42, 318000, tzinfo=tz),
        #      dt.datetime(2018, 8, 20, 12, 0, 56, 424000, tzinfo=tz)]



        print("len(x): {}".format(len(x)))
        np.random.seed(seed=1)
        y = np.random.random(len(x)) * 12 - 3

    #curve = self.p1.plot(self.data1)
   
    plot1D = Plot1D(backend=BACKEND)
    #plot1D.setGraphGrid("both")
    plot1D.setGraphGrid("both")
    xAxis = plot1D.getXAxis()
    xAxis.setLimits(-11, 35)

    if True and CREATE_DATES:
        xAxis.setTickMode(TickMode.TIME_SERIES)
        #xAxis.setTimeZone("UTC")

    logger.critical("timeZone: {}".format(xAxis.getTimeZone()))
    logger.critical("tickMode: {}".format(xAxis.getTickMode()))

    curve = plot1D.addCurve(x=x, y=y, legend='curve')

    logger.critical("timeZone: {}".format(xAxis.getTimeZone()))
    logger.critical("tickMode: {}".format(xAxis.getTickMode()))

    return plot1D



def makePlot2D():
    plot = Plot2D(backend=BACKEND)

    colormap = Colormap(name='viridis', normalization='linear', vmin=0.0, vmax=2.0)
    plot.setDefaultColormap(colormap)

    data = np.random.random(512 * 512).reshape(512, -1)
    plot.addImage(data, legend='random', replace=False)

    data = np.arange(512 * 512.).reshape(512, -1)
    plot.addImage(data, legend='arange', replace=False, origin=(512, 512))
    
    return plot





def makeHistogram():

    np.random.seed(seed=1)
    data = np.random.normal(size=10000)# * 12 - 3

    counts, binEdges = np.histogram(data)
    logger.debug("counts: {}".format(len(counts)))
    logger.debug("binEdges: {}".format(len(binEdges)))

    plot1D = Plot1D(backend=BACKEND)


    curve = plot1D.addCurve(x=binEdges, y=counts, histogram='left')

    logger.debug("curve; {}".format(curve))

    return plot1D




class MyWin(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super(MyWin, self).__init__(parent=parent)

        # Layout with legend
        self.contentsLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.contentsLayout)

        self.plot = makePlot1D()
        self.contentsLayout.addWidget(self.plot)

        self.myTestAction = QtWidgets.QAction("My test", self, triggered=self.myTest)
        self.myTestAction.setShortcut("Ctrl+T")
        self.addAction(self.myTestAction)


    def myTest(self):
        logger.info("Called my test")

        y = np.random.normal(size=int(1e5)) + 5

        x = np.arange(len(y)) + 9000

        self.plot.addCurve(x=x, y=y, color='red', legend='test curve')


    

def main():
    fmt = '%(filename)25s:%(lineno)-4d : %(levelname)-7s: %(message)s'
    logging.basicConfig(level='DEBUG', format=fmt)
    logger.info("START")
    print("START------------------------------------------------")


    app = QtWidgets.QApplication([])
    #win = makePlot2D()
    win = MyWin()
    #win = makeHistogram()
    win.show()



    app.exec_()
    
    
    
if __name__ == "__main__":
    main()

