"""
Module for custom Egress splash screen

Created on Jul 23, 2018

@author: simonr
"""

import os
# import random
import threading
import time

from PyQt5.QtGui        import QPixmap
from PyQt5.QtWidgets    import QApplication, QSplashScreen
from PyQt5.QtCore       import Qt

__all__ = ['TraySplashScreen']

class TraySplashScreen(QSplashScreen):
    """ Custom Egress splash screen. """

    def __init__(self):
        """ Create a new splash screen. The splash screen is actived from a separate thread, in
        order to allow the main application to continue to start, and also not to disappear too
        quickly. """

        # Load the splash screen image
        modulePath = os.path.dirname(os.path.abspath(__file__))
        self.pixmap = QPixmap(modulePath + '/images/splash.png')
        super().__init__(self.pixmap)
        self.setMask(self.pixmap.mask())
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        self._displayThread = None
        self._duration      = None

    def start(self, duration):
        """ Start displaying the splash screen for the given duration (in seconds). """
        self._duration      = duration
        self._displayThread = threading.Thread(target=self.show)
        self._displayThread.start()

    def show(self):
        """ Method which activates the screen, and closes it after the given duration. """
        super().show()
        # users      = ['Bert-Joost', 'Dick', 'Hiroki', 'Luciano', 'Kevin', 'Patrick']
        # developers = ['Jens', 'Martin', 'Pepijn', 'Shubham', 'Simon']

        # Below section displays a greeting from a developer to frequent users.
        # self.showMessage("<h1><font color='green'>Welcome {}!</font></h1><br />"
        #                  "<h2><font color='red'>Greetings from {}.</font></h2>".format(
        #                      random.choice(users), random.choice(developers)),
        #                  Qt.AlignVCenter | Qt.AlignCenter, Qt.red)

        # process qt events for the duration of the display (required, otherwise the screen
        # is not visible)
        splashStart = time.time()
        while time.time() - splashStart < self._duration:
            QApplication.processEvents()
            time.sleep(0.01)
        self.close()
